/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2010-2011 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * http://glassfish.java.net/public/CDDL+GPL_1_1.html
 * or packager/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at packager/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package com.sun.jersey.samples.atomserver.resources;

import com.sun.jersey.spi.MessageBodyWorkers;
import com.sun.syndication.feed.atom.Entry;
import com.sun.syndication.feed.atom.Feed;
import com.sun.syndication.io.FeedException;
import java.io.IOException;
import java.net.URI;
import java.util.Date;
import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.HttpHeaders;
import javax.ws.rs.core.UriBuilder;
import javax.ws.rs.core.UriInfo;


/**
 *
 * @author Paul.Sandoz@Sun.Com
 */
public class EditEntryResource extends EntryResource {
    protected UriInfo uriInfo;
    
    protected MessageBodyWorkers bodyContext;
    
    public EditEntryResource(String entryId, UriInfo uriInfo, 
            MessageBodyWorkers bodyContext) {
        super(entryId);
        this.uriInfo = uriInfo;
        this.bodyContext = bodyContext;
    }
    
    @PUT
    public Entry putEntry(Entry e) throws IOException, FeedException {
        UriBuilder entryUri = uriInfo.getBaseUriBuilder().
                path(FeedResource.class).
                path(entryId);
        AtomStore.updateLink(e, "self", entryUri.build());
        
        UriBuilder editEntryUri = getUriBuilder();
        AtomStore.updateLink(e, "edit", editEntryUri.build());
        
        if (AtomStore.hasMedia(entryId)) {
            // TODO update content link
            URI mediaUri = entryUri.path("media").build();
            
            URI editMediaUri = editEntryUri.path("media").build();
            AtomStore.updateLink(e, "edit-media", editMediaUri);
        }
        e.setId(entryId);
        e.setForeignMarkup(null);

        // Write out the entry document 
        AtomStore.createEntryDocument(bodyContext,  entryId, e);

        // Update the feed document with the entry
        Feed f = AtomStore.getFeedDocument();
        AtomStore.updateFeedDocumentWithExistingEntry(bodyContext, f, e);
        
        return e;
    }

    
    @DELETE
    public void deleteEntry() throws IOException, FeedException {
        AtomStore.deleteEntry(entryId);
        
        Feed f = AtomStore.getFeedDocument();
        AtomStore.updateFeedDocumentRemovingEntry(bodyContext, f, entryId);
    }
    
    
    @PUT
    @Path("media")
    @Consumes("*/*")
    public void putMedia(@Context HttpHeaders headers,
            byte[] update) throws IOException, FeedException {
        // Check if media exists, otherwise 404
        String mediaPath = AtomStore.getMediaPath(entryId);
        AtomStore.checkExistence(mediaPath);

        // Get the atom entry
        Feed f = AtomStore.getFeedDocument();
        Entry e = AtomStore.findEntry(entryId, f);
        
        // Update the entry
        e.setUpdated(new Date());
        UriBuilder editMediaEntryUri = getUriBuilder();
        AtomStore.updateLink(e, "edit-media", 
                editMediaEntryUri.build());
        editMediaEntryUri.path("..");
        AtomStore.updateLink(e, "edit", 
                editMediaEntryUri.build().normalize());
        
        // TODO update the content type
        
        // Write out the feed document
        AtomStore.updateFeedDocument(bodyContext, f);
        // Write out the entry document
        AtomStore.createEntryDocument(bodyContext, entryId, e);
        // Write out the media
        AtomStore.createMediaDocument(entryId, update);
    }
        
    protected UriBuilder getUriBuilder() {
        return uriInfo.getAbsolutePathBuilder();
    }
}
